(function() {
/* VARIABLES */
var board = null;
var game = null;
var pgnGame = null;
var boardflipped = false;
var promotionSource = null;
var promotionTarget = null;
var autorefresh = false;
var evtSource = null;
var autoflipTimer = null;
var justRefreshed= false;

/* EVENT SOURCE */
function addEvtMessage(event, message) {
	var date = new Date();
	var time = '';
	var h = date.getHours();
	if(h < 10) { time += '0'; }
	time += h+ ':';
	var m = date.getMinutes();
	if(m < 10) { time += '0'; }
	time += m+ ':';
	var s = date.getSeconds();
	if(s < 10) { time += '0'; }
	time += s;
	var html = '<div><span class="time">' + time + '</span><span class="'+event+'">'+message+'</span></div>';
	$('#messages').prepend(html);
}

function clearEvtMessages() {
        $('#messages').html('');
}

function startEventSource() {
        if(autorefresh) return;
        clearEvtMessages();
	$('#connection').removeClass("disconnected");
	$('#connection').removeClass("connected");
	$('#connection').addClass("unknown");
	autorefresh = true;

	var evtSource = new EventSource("../api/event-stream");
	evtSource.onmessage = function(event) { addEvtMessage('message', event.data); };
	evtSource.onerror= function(event) {
                console.log('event error');
                if(autorefresh) {
                    addEvtMessage('error', 'Event Source failed. (api/event-stream)');
                    autorefresh = false;
                }
		
		$('#connection').addClass("disconnected");
		$('#connection').removeClass("connected");
		$('#connection').removeClass("unknown");
		evtSource.close();
		
	 };
         evtSource.addEventListener("refreshGame", function(event) {
		console.log("event refreshGame");
                $('#connection').removeClass("disconnected");
                $('#connection').addClass("connected");
                $('#connection').removeClass("unknown");
		var parsed = JSON.parse(event.data);
                game = new Chess(parsed.fen);
                pgnGame = new PgnGame(game, parsed.tags, parsed.moves);
                pgnGame.gotoTreeNumber(parsed.treenumber);
		justRefreshed = true;
                downloadAndUpdateConfig();
	});
	evtSource.addEventListener("showHintMove", function(event) {
		var parsed = JSON.parse(event.data);
		console.log("showHintMove");
		console.log(parsed);
                addEvtMessage('showHintMove',parsed.move + ' ' + parsed.result.bestcp);
	});
	evtSource.addEventListener("listGames", function(event) {
		var parsed = JSON.parse(event.data);
		console.log("listGames");
		console.log(parsed);
	});
	evtSource.addEventListener("listGamesSheets", function(event) {
		var parsed = JSON.parse(event.data);
		console.log("listGamesSheets");
		console.log(parsed);
	});
	evtSource.addEventListener("listEngines", function(event) {
		var parsed = JSON.parse(event.data);
		console.log("listEngines");
		console.log(parsed);
	});
	evtSource.addEventListener("refresh", function(event) {
		console.log("event refresh " + event.data);
		reload(event.data);
	});
	evtSource.addEventListener("playedMove", function(event) {
		console.log('event playedMove ' + event.data);
                var parsed = JSON.parse(event.data);
                pgnGame.playedMove(parsed);
                updateStatus();
	});
	evtSource.addEventListener("draw", function(event) {
		console.log('event draw');
		board.position(game.fen(), !justRefreshed || !$('#autoflip').is(':checked'));
		updateStatus();
		justRefreshed = false;
	});
	evtSource.addEventListener("gotoTreeNumber", function(event) {
		console.log("event gotoTreeNumber " + event.data);
                pgnGame.gotoTreeNumber(event.data);
	});
        evtSource.addEventListener("closed", function(event) {
		console.log("event closed " + event.data);
                addEvtMessage('error', 'Connection closed.');
                autorefresh = false;
        });
}

/* FUNCTIONS */
function updateToMove() {
	if ((game.turn() === 'b' && board.orientation() === 'white') || (game.turn() === 'w' && board.orientation() === 'black')) {
		$('#toponmove i').show();
		$('#bottomonmove i').hide();
	} else {
       		$('#toponmove i').hide();
		$('#bottomonmove i').show();
	}
}

function updateConfig(data) {
	//boardflipped = data.board._computed.flipped;
	//board.orientation(boardflipped ? 'black' : 'white');
	updateToMove();
	if(data.console.playsounds) {
		$('#switchsounds i').addClass('fa-volume-up');
		$('#switchsounds i').removeClass('fa-volume-mute');
	}
	else {
		$('#switchsounds i').addClass('fa-volume-mute');
		$('#switchsounds i').removeClass('fa-volume-up');
	}
	$('#seconds').val(parseInt(data.engine.goseconds));
	$('#level').val(parseInt(data.engine._computed.level));
}

function downloadAndUpdateConfig() {
	$.get("../api/config", function (data) {
		updateConfig(data);
	});
}

function reload(fen) {
	game.load(fen);
       	board.position(fen);
	updateStatus();
}

function simpleCommand(cmd, args, donotreload) {
	hidePromotion();
	$.get("../api/command/"+cmd + '?args=' + (args ? args : ""), function () {
        	if(!donotreload) {
       			$.get("../api/fen", reload);
		}
	});
}

function updateStatus() {
	var status = '';

	var moveColor = 'White';
	if (game.turn() === 'b') {
		moveColor = 'Black';
	}
                    
	updateToMove();
	if (game.in_checkmate()) {
		status = 'Check mate!';
	}
	else if (game.in_draw()) {
		status = 'Drawn!';
	}
	else {
		status = '';
		if (game.in_check()) {
			status = 'Check!';
		}
	}
	$('#status').html(status);
	$('#fen').html(game.fen())

	clearTimeout(autoflipTimer);
	if($('#autoflip').is(':checked')) {
		if (!game.in_checkmate() && !game.in_draw()) {
			if((moveColor == 'Black' && !boardflipped) || (moveColor == 'White' && boardflipped)) {
		autoflipTimer = setTimeout(flipboard, justRefreshed ? 0 : 500);
			}
		}
	}
}

function flipboard() {
	board.flip();
	boardflipped = !boardflipped;
	//downloadAndUpdateConfig();
	updateToMove();
}

function hidePromotion() {
	$('#promotionPieces_w').hide();
	$('#promotionPieces_b').hide();
}
            
function promote(piece) {
	hidePromotion();
	var move = game.move({
		from: promotionSource,
		to: promotionTarget,
		promotion: piece
	});

	// illegal move
	if (move === null) {
		return;
	}
	board.position(game.fen());
	$.get("../api/command/move?args=" + promotionSource + promotionTarget + piece, function () {});
	updateStatus();
}


/* EVENTS */

$('#send').on('submit', function(event) {
	event.preventDefault();
	hidePromotion();
       	var cmd = $('#cmd').val();
	$('#cmd').val('');
	var cmdArr = cmd.split(" ",1);
	var args = cmd.substring(cmdArr[0].length);
	$.get("../api/command/"+cmdArr[0]+"?args=" +  args, function () {
		$.get("../api/fen", function (fen) {
 	              reload(fen);
		      downloadAndUpdateConfig();
		});
	});
	return false;
});

/* disconnect */
$('#disconnect').on('click', function() {
            simpleCommand('stop', '', true);
            $('#connection').addClass("disconnected");
            $('#connection').removeClass("connected");
});

/* right menu actions */
$('#new').on('click', function() {
	simpleCommand('new');
});
$('#refresh').on('click', function() {
        if(!autorefresh) {
            startEventSource();
        } else {
            simpleCommand('board');
        }
	downloadAndUpdateConfig();
});
$('#flip').on('click', function() {
	//simpleCommand('flipboard');
	flipboard();
	
});
$('#switchsounds').on('click', function() {
	simpleCommand('switchsounds');
	downloadAndUpdateConfig();
});
/* bottom naviagion */
$('#backward').on('click', function() {
	simpleCommand('uu');
});
$('#step-backward').on('click', function() {
	simpleCommand('undo');
});
$('#step-forward').on('click', function() {
	simpleCommand('redo');
});
$('#forward').on('click', function() {
	simpleCommand('redo', '1000');
});
$('#play').on('click', function() {
	simpleCommand('play');
	$('#countdown').html($('#seconds').val());
	var handler = setInterval(function() {
		var s = $('#countdown').text();
		if(!s || s == '0') {
			clearInterval(handler);
			$('#countdown').html('');
			return;
		}
		$('#countdown').html(parseInt(s)-1);

	}, 1000);
});
$('#seconds').on('change', function() {
	simpleCommand('seconds', $(this).val());
});
$('#level').on('change', function() {
	simpleCommand('level', $(this).val());
});

/* promotion */
$('#wqueen').on('click', function()  {promote('q');});
$('#wrook').on('click', function()   {promote('r');});
$('#wbishop').on('click', function() {promote('b');});
$('#wknight').on('click', function() {promote('n');});
$('#bqueen').on('click', function()  {promote('q');});
$('#brook').on('click', function()   {promote('r');});
$('#bbishop').on('click', function() {promote('b');});
$('#bknight').on('click', function() {promote('n');});


/* EXECUTE */
            
$.get("../version", function (version) {
	$("#version").html(version);
});


$.get("../api/version", function (version) {
	$("#apiversion").html(version);
});

 
$.get("../api/fen", function (fen) {
                function onDragStart(source, piece, position, orientation) {
                    // do not pick up pieces if the game is over
                    if (game.game_over()) {
                        return false;
                    }
                    // only pick up pieces for the side to move
                    if ((game.turn() === 'w' && piece.search(/^b/) !== -1) ||
                            (game.turn() === 'b' && piece.search(/^w/) !== -1)) {
                        return false;
                    }
                }
                
                function promotionMove(source, target, color) {
                    promotionSource = source;
                    promotionTarget = target;
                    var h = ($('#board').height()-60)/2;
                    $('#promotionPieces_'+color).css('margin-top', h+'px');
                    $('#promotionPieces_'+color).show();
                }

                function onDrop(source, target) {
                    // see if the move is legal
                    var piece = game.get(source);
                    var move = game.move({
                        from: source,
                        to: target,
                        promotion: 'q'
                    });

                    // illegal move
                    if (move === null) {
                        return 'snapback';
                    }
                    
                    var promote = '';
                    if(piece.type == 'p' && (target[1] == '8' || target[1] == '1')) {
                        if($('#promoteToQueen').is(':checked')) {
                            promote = 'q';
                        } else {
                            game.undo();
                            promotionMove(source, target, piece.color);
                            return;
                        }
                    }
                    
                    $.get("../api/command/move?args=" + source + target + promote, function () {
                    });
                    updateStatus();
                }

                // update the board position after the piece snap
                // for castling, en passant, pawn promotion
                function onSnapEnd() {
                    board.position(game.fen());
                }
                var config = {
                    draggable: true,
                    position: fen,
                    onDragStart: onDragStart,
                    onDrop: onDrop,
                    onSnapEnd: onSnapEnd,
		    pieceTheme: 'chessboardjs/img/chesspieces/wikipedia/{piece}.png'
                }
                board = Chessboard('board', config);
                $(window).resize(board.resize);
		evtSource = startEventSource();
});

})();
